<?php
/**
 * Copyright 2021-2023 InPost S.A.
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the EUPL-1.2 or later.
 * You may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/software/page/eupl
 * It is also bundled with this package in the file LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Licence is distributed on an AS IS basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions
 * and limitations under the Licence.
 *
 * @author    InPost S.A.
 * @copyright 2021-2023 InPost S.A.
 * @license   https://joinup.ec.europa.eu/software/page/eupl
 */

namespace InPost\Shipping\Hook;

use InPost\Shipping\Configuration\CheckoutConfiguration;
use InPost\Shipping\GeoWidget\GeoWidgetTokenProvider;
use InPost\Shipping\HookUpdater;
use InPost\Shipping\ShipX\Resource\Service;
use Media;
use ModuleFrontController;
use Tools;

class Assets extends AbstractHook
{
    protected const HOOK_LIST = [
        'actionAdminControllerSetMedia',
        'actionFrontControllerSetMedia',
        'displayAdminAfterHeader',
    ];

    const SUPERCHECKOUT_MODULE = 'supercheckout';
    const STEASYCHECKOUT_MODULE = 'steasycheckout';
    const THECHECKOUT_MODULE = 'thecheckout';
    const ETS_ONEPAGECHECKOUT_MODULE = 'ets_onepagecheckout';

    protected $ordersDisplay;

    public function hookActionAdminControllerSetMedia()
    {
        $this->updateHookRegistrations();

        if (Tools::getValue('controller') === 'AdminOrders') {
            $display = $this->getOrdersDisplay();

            Media::addJsDef([
                'inPostLockerServices' => Service::LOCKER_SERVICES,
                'inPostLockerStandard' => Service::INPOST_LOCKER_STANDARD,
            ]);

            if ($display === 'view') {
                $assetsManager = $this->module->getAssetsManager();

                $assetsManager
                    ->registerJavaScripts([
                        'admin/tools.js',
                        'admin/common.js',
                        'admin/order-details.js',
                    ])
                    ->registerStyleSheets([
                        'admin/orders.css',
                    ]);

                $this->setGeoWidgetMedia();
            } elseif ($display === 'index') {
                $this->module
                    ->getAssetsManager()
                    ->registerJavaScripts([
                        'admin/tools.js',
                        'admin/order-list.js',
                    ]);
            }
        }

        if ($this->shouldDisplayLoader()) {
            $this->module
                ->getAssetsManager()
                ->registerStyleSheets(['admin/loader.css']);
        }
    }

    public function hookDisplayAdminAfterHeader()
    {
        return $this->shouldDisplayLoader()
            ? $this->module->display($this->module->name, 'views/templates/hook/loader.tpl')
            : '';
    }

    protected function shouldDisplayLoader(): bool
    {
        return Tools::getValue('controller') === 'AdminOrders'
            && in_array($this->getOrdersDisplay(), ['index', 'view'])
            || isset($this->context->controller->module)
            && $this->context->controller->module === $this->module;
    }

    public function hookActionFrontControllerSetMedia()
    {
        if ($this->isCheckoutControllerContext()) {
            $assetsManager = $this->module->getAssetsManager();

            $assetsManager
                ->registerJavaScripts([
                    'checkout.js',
                ])
                ->registerStyleSheets([
                    'front.css',
                ]);

            if ($scripts = $this->getModuleSpecificScriptFiles()) {
                $assetsManager->registerJavaScripts($scripts);
            }

            Media::addJsDef([
                'inPostAjaxController' => $this->context->link->getModuleLink($this->module->name, 'ajax'),
            ]);

            $this->setGeoWidgetMedia();
        }
    }

    protected function getOrdersDisplay(): string
    {
        if (!isset($this->ordersDisplay)) {
            $this->ordersDisplay = $this->initOrdersDisplay();
        }

        return $this->ordersDisplay;
    }

    protected function initOrdersDisplay(): string
    {
        switch (Tools::getValue('action')) {
            case 'vieworder':
                return 'view';
            case 'addorder':
                return 'create';
            default:
                return 'index';
        }
    }

    protected function isCheckoutControllerContext(): bool
    {
        $controller = Tools::getValue('controller');

        if (in_array($controller, ['order', 'orderopc'])) {
            return true;
        }

        if ($this->context->controller instanceof ModuleFrontController) {
            switch ($this->context->controller->module->name) {
                case self::SUPERCHECKOUT_MODULE:
                    return 'supercheckout' === $this->getModuleControllerName($controller);
                case self::STEASYCHECKOUT_MODULE:
                    return 'default' === $this->getModuleControllerName($controller);
                case self::THECHECKOUT_MODULE:
                    return 'front' === $this->getModuleControllerName($controller);
                case self::ETS_ONEPAGECHECKOUT_MODULE:
                    return 'order' === $this->getModuleControllerName($controller);
                default:
                    return $this->isCustomCheckoutController(
                        $this->context->controller->module->name,
                        $controller
                    );
            }
        }

        return false;
    }

    protected function getModuleControllerName(string $controller): string
    {
        $parts = explode('-', $controller);

        return end($parts);
    }

    protected function isCustomCheckoutController(string $moduleName, string $controller): bool
    {
        /** @var CheckoutConfiguration $configuration */
        $configuration = $this->module->getService('inpost.shipping.configuration.checkout');

        if ($configuration->isUsingCustomCheckoutModule()) {
            $controllers = $configuration->getCustomCheckoutControllers();

            return isset($controllers[$moduleName])
                && in_array(
                    $this->getModuleControllerName($controller),
                    $controllers[$moduleName]
                );
        }

        return false;
    }

    protected function getModuleSpecificScriptFiles(): array
    {
        if ($this->context->controller instanceof ModuleFrontController) {
            switch ($this->context->controller->module->name) {
                case self::SUPERCHECKOUT_MODULE:
                    return ['modules/supercheckout.js'];
                case self::STEASYCHECKOUT_MODULE:
                    return ['modules/steasycheckout.js'];
                case self::THECHECKOUT_MODULE:
                    return ['modules/thecheckout.js'];
                default:
                    return [];
            }
        }

        return [];
    }

    protected function updateHookRegistrations()
    {
        /** @var HookUpdater $updater */
        $updater = $this->module->getService('inpost.shipping.hook_updater');

        $updater->updateHookRegistrations();
    }

    protected function setGeoWidgetMedia()
    {
        /** @var GeoWidgetTokenProvider $tokenProvider */
        $tokenProvider = $this->module->getService('inpost.shipping.geo_widget.token_provider');

        if ($token = $tokenProvider->getToken()) {
            $this->module
                ->getAssetsManager()
                ->registerGeoWidgetAssets($token->isSandbox())
                ->registerJavaScripts(['geowidget_v5.js']);

            Media::addJsDef([
                'inPostGeoWidgetToken' => $token->getToken(),
                'inPostLanguage' => Tools::strtolower($this->context->language->iso_code) === 'pl' ? 'pl' : 'en',
            ]);
        }
    }
}
