<?php
/**
 * Copyright 2021-2023 InPost S.A.
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the EUPL-1.2 or later.
 * You may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/software/page/eupl
 * It is also bundled with this package in the file LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Licence is distributed on an AS IS basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions
 * and limitations under the Licence.
 *
 * @author    InPost S.A.
 * @copyright 2021-2023 InPost S.A.
 * @license   https://joinup.ec.europa.eu/software/page/eupl
 */

namespace InPost\Shipping\Hook;

use Address;
use Carrier;
use Cart;
use Customer;
use InPost\Shipping\CartChoiceUpdater;
use InPost\Shipping\Presenter\CheckoutDataPresenter;
use InPost\Shipping\Traits\ErrorsTrait;
use InPostCarrierModel;
use InPostCartChoiceModel;
use ModuleFrontController;
use Order;
use Tools;

class Checkout extends AbstractHook
{
    use ErrorsTrait;

    protected const HOOK_LIST = [
        'actionCarrierProcess',
        'actionValidateOrder',
    ];

    protected const HOOK_LIST_16 = [
        'displayCarrierList',
    ];

    protected const HOOK_LIST_17 = [
        'displayCarrierExtraContent',
        'actionValidateStepComplete',
    ];

    protected $deliveryChoiceProcessed = false;

    public function hookActionCarrierProcess(array $params)
    {
        if ($this->deliveryChoiceProcessed || !Tools::isSubmit('delivery_option')) {
            return;
        }

        $this->processDeliveryChoice(
            $params['cart'],
            Tools::getAllValues(),
            $this->context->controller instanceof ModuleFrontController
        );
    }

    public function hookActionValidateStepComplete(array $params)
    {
        if ('delivery' !== $params['step_name']) {
            return;
        }

        $deliveryChoiceProcessed = $this->processDeliveryChoice(
            $this->context->cart,
            $params['request_params'],
            $this->context->controller instanceof ModuleFrontController
        );

        if (!$deliveryChoiceProcessed) {
            return;
        }

        $this->deliveryChoiceProcessed = true;

        if ($this->hasErrors()) {
            $params['completed'] = false;
            $this->storeSessionData($params['request_params']);
        }
    }

    protected function processDeliveryChoice(Cart $cart, array $requestParams, bool $addControllerErrors = true): bool
    {
        $deliveryOption = isset($requestParams['delivery_option'])
            ? $requestParams['delivery_option']
            : $this->context->cart->getDeliveryOption(null, true);

        if (!isset($deliveryOption[$cart->id_address_delivery])) {
            return false;
        }

        $carrierIds = explode(',', trim($deliveryOption[$cart->id_address_delivery], ','));
        foreach ($carrierIds as $carrierId) {
            if ($carrierData = InPostCarrierModel::getDataByCarrierId($carrierId)) {
                $updater = $this->getCartChoiceUpdater($cart->id, $carrierData)
                    ->setEmail($requestParams['inpost_email'] ?? null)
                    ->setPhone($requestParams['inpost_phone'] ?? null);

                if ($carrierData['lockerService']) {
                    $locker = $requestParams['inpost_locker'][$carrierId] ?? null;

                    $updater->setTargetPoint($locker);
                }

                $updater->saveChoice($cart->id);

                if ($updater->hasErrors()) {
                    $this->setErrors($errors = $updater->getErrors());

                    if ($addControllerErrors) {
                        $this->context->controller->errors = array_merge(
                            $this->context->controller->errors,
                            array_values($errors)
                        );
                    }
                }

                return true;
            }
        }

        return false;
    }

    protected function getCartChoiceUpdater(int $cartId, array $carrierData): CartChoiceUpdater
    {
        /** @var CartChoiceUpdater $updater */
        $updater = $this->module->getService('inpost.shipping.updater.cart_choice');

        return $updater
            ->setCartChoice(new InPostCartChoiceModel($cartId))
            ->setCarrierData($carrierData);
    }

    public function hookDisplayCarrierExtraContent(array $params)
    {
        if ((!Tools::getValue('confirmDeliveryOption') || $this->deliveryChoiceProcessed) &&
            $carrierData = InPostCarrierModel::getDataByCarrierId($params['carrier']['id'])
        ) {
            $this->assignTemplateVariables($carrierData, $this->retrieveSessionData());

            return $this->module->display(
                $this->module->name,
                'views/templates/hook/carrier-extra-content.tpl'
            );
        }

        return '';
    }

    protected function assignTemplateVariables(array $carrierData, array $sessionData = [])
    {
        /** @var CheckoutDataPresenter $presenter */
        $presenter = $this->module->getService('inpost.shipping.presenter.checkout_data');

        $this->context->smarty->assign($presenter->present($carrierData, $sessionData));
    }

    public function hookActionValidateOrder(array $params)
    {
        /** @var Order $order */
        $order = $params['order'];
        $carrier = new Carrier($order->id_carrier);
        $cartChoice = new InPostCartChoiceModel($order->id_cart);

        if ($carrier->external_module_name !== $this->module->name) {
            $cartChoice->delete();
        } elseif (empty($cartChoice->email) || empty($cartChoice->phone)) {
            $this->updateCartChoiceByOrderData($cartChoice, $order);
        }
    }

    // preserve errors and submitted values to retrieve after redirect
    protected function storeSessionData(array $requestParams)
    {
        $data = json_encode([
            'email' => $requestParams['inpost_email'] ?? null,
            'phone' => $requestParams['inpost_phone'] ?? null,
            'errors' => $this->getErrors(),
        ]);

        switch (session_status()) {
//            case PHP_SESSION_NONE:
//                session_start();
//                // no break
//            case PHP_SESSION_ACTIVE:
//                $_SESSION['inpost_data'] = $data;
//                break;
            default:
                $this->context->cookie->inpost_data = $data;
                break;
        }
    }

    protected function retrieveSessionData()
    {
        static $data;

        if (!isset($data)) {
            $data = [];

            switch (session_status()) {
//                case PHP_SESSION_NONE:
//                    session_start();
//                    // no break
//                case PHP_SESSION_ACTIVE:
//                    if (isset($_SESSION['inpost_data'])) {
//                        $data = json_decode($_SESSION['inpost_data'], true);
//                        unset($_SESSION['inpost_data']);
//                    }
//                    break;
                default:
                    if (isset($this->context->cookie->inpost_data)) {
                        $data = json_decode($this->context->cookie->inpost_data, true);
                        unset($this->context->cookie->inpost_data);
                    }
                    break;
            }
        }

        return $data;
    }

    private function updateCartChoiceByOrderData(InPostCartChoiceModel $cartChoice, Order $order)
    {
        if (empty($cartChoice->service)) {
            if (null === $data = InPostCarrierModel::getDataByCarrierId($order->id_carrier)) {
                return;
            }

            $cartChoice->service = $data['service'];
        }

        if (empty($cartChoice->phone)) {
            $address = new Address($order->id_address_delivery);
            $cartChoice->phone = $address->phone_mobile ?: $address->phone;
        }

        if (empty($cartChoice->email)) {
            $customer = new Customer($order->id_customer);
            $cartChoice->email = $customer->email;
        }

        if (empty($cartChoice->id)) {
            $cartChoice->id = (int) $order->id_cart;
            $cartChoice->add();
        } else {
            $cartChoice->update();
        }
    }
}
